// Configuration
const width = 1000;
const height = 700;

// Chargement des données
d3.csv('./data/2014.csv').then(initialize);

/**
 * Initialise la visualisation
 *
 * @param data      Données à utiliser
 */
function initialize(data) {
  // Traitement des données
  const zones = [];
  data = data.map(d => {
    if (zones.indexOf(d.zone) === -1) {
      zones.push(d.zone);
    }
    // On conserve uniquement les éléments "name", "population" et "zone" sur les données
    return {
      name: d.name,
      population: parseInt(d.population),
      zone: d.zone
    };
  });
  zones.sort();

  // Définitions de l'échelle de couleur
  const colors = d3.scaleOrdinal(d3.schemePastel1)
    .domain(zones);

  // Création de la liste déroulante
  d3.select('select')
    .on('change', function() {
      const zone = d3.select(this).property('value');
      if (zone === 'Monde') {
        update(data)
      } else {
        update(data.filter(e => e.zone === zone));
      }
    })
    .selectAll('option')
    .data(['Monde', ...zones])
    .enter()
    .append('option')
    .attr('value', d => d)
    .text(d => d);

  // Création de l'élément SVG
  const svg = d3.select('svg')
    .attr('width', width)
    .attr('height', height);

  // Initialisation du "pack"
  const pack = d3.pack()
    .size([width, height])
    .padding(3);

  update(data);

  /**
   * Permet la mise à jour de la visualisation en fonction des données spécifiées.
   *
   * @param data    Les données à utiliser pour dessiner la visualisation
   */
  function update(data) {
    const parent = d3.hierarchy({children: data}).sum(d => d.population);
    const root = pack(parent);

    const groups = svg.selectAll('g')
      .data(root.leaves());

    // TODO
    // 1) Utiliser les mécanismes de mise à jour pour dessiner les textes et les cercles à l'intérieur des groupes.
    // 2) Ajouter un élément "title" à l'intérieur des cercles pour afficher le nom du pays ainsi que sa population.
    // 3) Utiliser l'échelle de couleurs (colors) pour colorier les cercles selon leur zone dans le monde.
    // 4) Afficher la taille du nom d'un pays de manière proportionnelle au rayon du cercle qui lui est associé.
    // 5) Ajouter une transition de 500 ms pour communiquer un changement dans les données.

    const groupsCreated = groups.enter()
      .append('g')
      .attr('transform', `translate(${width * 0.5}, ${height * 0.5})`);

    groupsCreated.append('circle')
      .append('title');

    groupsCreated.append('text');

    groups.merge(groupsCreated)
      .transition()
      .duration(500)
      .attr('transform', d => `translate(${d.x}, ${d.y})`);

    groups.merge(groupsCreated)
      .select('circle')
      .transition()
      .duration(500)
      .attr('r', d => d.r)
      .style('fill', d  => colors(d.data.zone))
      .select('title')
      .text(d => `${d.data.name} (${d3.format(".2s")(d.data.population)} d'habitants)`);

    groups.merge(groupsCreated)
      .select('text')
      .style('alignment-baseline', 'middle')
      .style('text-anchor', 'middle')
      .style('font-size', d => d.r * 0.3 + 'px')
      .text(d => {
        if (d.r > 15) {
          return d.data.name
        }
        return '';
      });

    groups.exit()
      .remove();
  }
}
